/*
 * 2D Game Engine
 * Game.cpp: Base Game engine class
 * Based on pikuma.com 2D game engine in C++ and Lua course
 * Copyright (c) 2021 986-Studio. All Right Reserved
 *
 * Created by Manoël Trapier on 09/02/2021.
 */
#include <Game.h>
#include <stdio.h>
#include <SDL.h>
#include <SDL_image.h>
#include <glm/glm.hpp>

Game::Game()
{
    this->isRunning = false;
    this->windowsHeight = 0;
    this->windowsWidth = 0;
}

Game::~Game()
{

}

void Game::Initialize()
{
    int ret;
    SDL_DisplayMode displayMode;

    ret = SDL_Init(SDL_INIT_EVERYTHING);
    if (ret)
    {
        fprintf(stderr, "SDL Initialisation error.\n");
        return;
    }

    SDL_GetCurrentDisplayMode(0, &displayMode);
    this->windowsWidth = displayMode.w;
    this->windowsHeight = displayMode.h;

    this->window = SDL_CreateWindow(NULL, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                                    this->windowsWidth, this->windowsHeight, SDL_WINDOW_BORDERLESS);
    if (!window)
    {
        fprintf(stderr, "SDL window creation error.\n");
        return;
    }

    this->renderer = SDL_CreateRenderer(window, -1, 0);
    if (!renderer)
    {
        fprintf(stderr, "SDL renderer creation error.\n");
        return;
    }

    SDL_SetWindowFullscreen(this->window, SDL_WINDOW_FULLSCREEN);

    this->isRunning = true;
}

void Game::Destroy()
{
    if (this->renderer)
    {
        SDL_DestroyRenderer(this->renderer);
    }
    if (this->window)
    {
        SDL_DestroyWindow(this->window);
    }

    SDL_Quit();
}

void Game::Run()
{
    this->Setup();

    while(this->isRunning)
    {
        this->ProcessInput();
        this->Update();
        this->Render();
    }
}

glm::vec2 playerPostion;
glm::vec2 playerVelocity;

void Game::Setup()
{
    playerPostion = glm::vec2(10, 20);
    playerVelocity = glm::vec2(100, 5);
}

void Game::Update()
{
    int32_t timeToWait = MILLISECS_PER_FRAME - (SDL_GetTicks() - this->millisecsPerviousFrame);
    if (timeToWait > 0 && timeToWait <= MILLISECS_PER_FRAME)
    {
        SDL_Delay(timeToWait);
    }
    double deltaTime = (SDL_GetTicks() - this->millisecsPerviousFrame) / 1000.0;

    this->millisecsPerviousFrame = SDL_GetTicks();

    playerPostion.x += (playerVelocity.x * deltaTime);
    playerPostion.y += (playerVelocity.y * deltaTime);
}

void Game::ProcessInput()
{
    SDL_Event event;

    while(SDL_PollEvent(&event))
    {
        switch(event.type)
        {
            case SDL_QUIT:
                this->isRunning = false;
                break;

            case SDL_KEYDOWN:
                if (event.key.keysym.sym == SDLK_ESCAPE)
                {
                    this->isRunning = false;
                }
                break;

            default:
                break;
        }
    }

}

void Game::Render()
{
    SDL_SetRenderDrawColor(this->renderer, 21, 21, 21, 255);
    SDL_RenderClear(this->renderer);

    SDL_Surface *surface = IMG_Load("assets/images/tank-tiger-right.png");
    SDL_Texture *texture = SDL_CreateTextureFromSurface(this->renderer, surface);
    SDL_FreeSurface(surface);

    SDL_Rect dest = { static_cast<int>(playerPostion.x), static_cast<int>(playerPostion.y), 32, 32 };

    SDL_RenderCopy(this->renderer, texture, NULL, &dest);

    SDL_DestroyTexture(texture);

    SDL_RenderPresent(this->renderer);
}
